/*
 * Copyright 2006 Google Inc. All Rights Reserved.
 * 
 * Author: Alden King
 */

package com.google.hadoop.examples;

import java.io.IOException;
import java.util.Iterator;
import java.util.ArrayList;

import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.io.FloatWritable;
import org.apache.hadoop.io.WritableComparable;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;
import org.apache.hadoop.mapred.SequenceFileInputFormat;
import org.apache.nutch.parse.Parse;
import org.apache.nutch.parse.ParseException;
import org.apache.nutch.parse.ParseUtil;
import org.apache.nutch.protocol.Content;

// This mapreduce performs scoring for web search.
// It reads in a repository from a nutch crawl and scores each document
// with each scorer initialized in the Mapper. The reducer sums up all the
// scores for a given url and outputs the sum with the url.
// A second mapreduce (SearchExampleSorter) reads in the scored urls and sorts the results.


public class SearchExample {
  
  public static void main(String[] args) throws IOException {
    JobConf conf = new JobConf(SearchExample.class);
    conf.setJobName("Map-reduce search scoring");
    
    // Configuration file for nutch data.
    conf.addDefaultResource("nutch-default.xml");
   
    // Input data type for Nutch crawl data.
    conf.setInputFormat(SequenceFileInputFormat.class);
    
   
    conf.setOutputKeyClass(Text.class);
    conf.setMapOutputValueClass(FloatWritable.class);
    conf.setOutputValueClass(FloatWritable.class);
    
    conf.setMapperClass(ExtractSegmentMapper.class);
    conf.setReducerClass(ExtractSegmentReducer.class);
    
    if (args.length < 3) {
      System.out.println("ERROR: Wrong number of parameters");
      System.out.println("simple <input_path> <output_path> <query_string>");
      System.exit(1);
    }
    
    conf.setInputPath(new Path(args[0]));
    conf.setOutputPath(new Path(args[1]));
    // Set the query string for the search.
    conf.set("queryString", args[2]);
   	
    JobClient.runJob(conf);
  }

  
  //The mapper should have a class that determines if the parsed HTML matches the query.
  //It should then iterate through a list of scorers or some such to output scores.
  public static class ExtractSegmentMapper extends MapReduceBase implements Mapper {
	  // Query string for the search.
	  private String query;
	  // Parse utility to read html.
	  private ParseUtil util;
	  // A list of objects that can emit scores for documents.
	  private ArrayList<DocumentScorer> scorers;
	  
	  // Add a SimpleScorer.
	  public ExtractSegmentMapper(){
		  this.scorers = new ArrayList<DocumentScorer>();
		  this.scorers.add(new SimpleScorer());
	  }
	  
	  // Configure the mapper for this map-reduce search.
	  // Set the query string for each scorer.
	  public void configure(JobConf conf) {
		  this.util = new ParseUtil(conf);
		  this.query = conf.get("queryString");
		  Iterator<DocumentScorer> it = this.scorers.iterator();
		  while(it.hasNext())
			  it.next().SetQuery(query);
	  }
    
	  // The map function to be called for each document.
	  public void map(WritableComparable key, Writable value,
			  		OutputCollector output, Reporter reporter)
      	throws IOException {

		  // Prepare the input data.
		  Content content = (Content)value;
		  Parse parse = null;

		  try {
			  // Try to parse the page.
			  parse = util.parse(content);
			  String page = parse.getText();
    		  
			  // Iterate over the document scorers and emit each non-zero score.
			  Iterator<DocumentScorer> it = this.scorers.iterator();
			  while(it.hasNext()) {
				  float score = it.next().ScoreDocument(page);
				  // Emit the score keyed by url.
				  if(score>0)
					  output.collect(new Text(content.getUrl()), new FloatWritable(score));
			  }
        
    	
      		} catch (ParseException e) {
      			// If we cannot parse the document then just skip it.
      			;
      		}
	  }
  }

  // This reducer sums the scores per url so that they can be ordered later.
  public static class ExtractSegmentReducer extends MapReduceBase implements Reducer {
    public void reduce(WritableComparable key, Iterator values,
                       OutputCollector output, Reporter reporter)
      throws IOException {
    	
    	float score = 0;
    	// For each score (value received), sum them together.
    	while (values.hasNext()){
    		float currentScore = ((FloatWritable)values.next()).get();
          	score += currentScore;          	
          }
    	// Output the summed score and url.
    	output.collect(new FloatWritable(score), new Text(key.toString()));
    }
  }
}








