package com.google.hadoop.examples;

import java.io.IOException;
import java.util.*;

import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.FloatWritable;
import org.apache.hadoop.io.Text;
import org.apache.hadoop.io.Writable;
import org.apache.hadoop.io.WritableComparable;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.MapReduceBase;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;


// This mapreduce reads in the (score, url) pairs found in the first mapreduce
// and re-emits them keyed by score. Hadoop automaticly sorts the keys, giving us 
// a list of urls ordered by scores - our list of web search results.


public class SearchExampleSorter{
  
  public static void main(String[] args) throws IOException {
    JobConf conf = new JobConf(LineIndexer.class);
    conf.setJobName("Search Example Sorter");
    
    // The keys are floats (document scores).
    conf.setOutputKeyClass(FloatWritable.class);
    // The values are urls.
    conf.setOutputValueClass(Text.class);
    
    conf.setMapperClass(SearchExampleSorter.SortMapper.class);
    conf.setReducerClass(SearchExampleSorter.SortReducer.class);

    if (args.length < 2) {
      System.out.println("ERROR: Wrong number of parameters");
      System.out.println("simple <input_path> <output_path>");
      System.exit(1);
    }
    
    conf.setInputPath(new Path(args[0]));
    conf.setOutputPath(new Path(args[1]));

    JobClient.runJob(conf);
  }

  // Find the (score, url) pair and output it keyed by score for sorting.
  public static class SortMapper extends MapReduceBase implements Mapper {
    
	  public void map(WritableComparable key, Writable val,
                    OutputCollector output, Reporter reporter)
      throws IOException {
		String line = ((Text)val).toString().trim();
    	int index = line.indexOf("\t");
    	float score = Float.parseFloat(line.substring(0, index));
    	String url = line.substring(index+1);
    	
    	output.collect(new FloatWritable(score), new Text(url));    	
	  }
}
  
  // The values will be automaticly sorted, so we just output them.
  public static class SortReducer extends MapReduceBase implements Reducer {
    public void reduce(WritableComparable key, Iterator values,
                       OutputCollector output, Reporter reporter)
      throws IOException {

      // Output each pair.
      while (values.hasNext()){
    	  Text url = (Text)values.next();
    	  output.collect(url, key);
      }
    	  
    }
  }
}
